@@ -31,6 +31,7 @@ import net.i2p.i2ptunnel.I2PTunnelServer;
 import net.i2p.i2ptunnel.TunnelController;
 import net.i2p.i2ptunnel.TunnelControllerGroup;
 import net.i2p.i2ptunnel.ui.GeneralHelper;
+import net.i2p.i2ptunnel.ui.Messages;
 import net.i2p.i2ptunnel.ui.TunnelConfig;
 import net.i2p.util.Addresses;
 import net.i2p.util.Log;
@@ -133,8 +134,11 @@ public class IndexBean {
         }
     }
 
-    /** do we know this nonce? @since 0.8.1 */
-    private static boolean haveNonce(String nonce) {
+    /**
+      * do we know this nonce?
+      * @since 0.8.1 public since 0.9.35
+      */
+    public static boolean haveNonce(String nonce) {
         synchronized (_nonces) {
             return _nonces.contains(nonce);
         }
@@ -263,6 +267,11 @@ public class IndexBean {
         return _t("Stopping tunnel") + ' ' + getTunnelName(_tunnel) + "...";
     }
     
+    /**
+     * Only call this ONCE! Or you will get duplicate tunnels on save.
+     *
+     * @return not HTML escaped, or "" if empty
+     */
     private String saveChanges() {
         // FIXME name will be HTML escaped twice
         return getMessages(_helper.saveTunnel(_tunnel, _config));
@@ -296,7 +305,7 @@ public class IndexBean {
             try {
                 String result = processAction();
                 if (result.length() > 0)
-                    buf.append(processAction()).append('\n');
+                    buf.append(result).append('\n');
             } catch (RuntimeException e) {
                 _log.log(Log.CRIT, "Error processing " + _action, e);
                 buf.append("Error: ").append(e.toString()).append('\n');
@@ -1204,12 +1213,8 @@ public class IndexBean {
     /** New key */
     private String generateNewEncryptionKey() {
         TunnelController tun = getController(_tunnel);
-        Properties config = getConfig();
         if (tun == null) {
             // creating new
-            tun = new TunnelController(config, "", true);
-            _group.addController(tun);
-            saveChanges();
         } else if (tun.getIsRunning() || tun.getIsStarting()) {
             return "Tunnel must be stopped before modifying leaseset encryption key";
         }
