@@ -175,7 +175,7 @@ public class ConfigUpdateHandler extends FormHandler {
             int proxyPort = proxyPort(_context);
             if (shouldProxy && proxyPort == ConfigUpdateHandler.DEFAULT_PROXY_PORT_INT &&
                 proxyHost.equals(ConfigUpdateHandler.DEFAULT_PROXY_HOST) &&
-                _context.portMapper().getPort(PortMapper.SVC_HTTP_PROXY) < 0) {
+                !_context.portMapper().isRegistered(PortMapper.SVC_HTTP_PROXY)) {
                 addFormError(_t("HTTP client proxy tunnel must be running"));
                 return;
             }
