@@ -278,8 +278,8 @@ class NetDbRenderer {
         buf.append("</table>\n");
 
         if (leases.isEmpty()) {
-          if (!debug)
-              buf.append("<div id=\"noleasesets\"><i>").append(_t("No Leasesets currently active.")).append("</i></div>");
+            //if (!debug)
+            //    buf.append("<div id=\"noleasesets\"><i>").append(_t("No Leasesets currently active.")).append("</i></div>");
         } else {
           if (debug) {
             // Find the center of the RAP leasesets
@@ -290,7 +290,7 @@ class NetDbRenderer {
             medianCount = rapCount / 2;
           }
 
-          boolean linkSusi = WebAppStarter.isWebAppRunning("susidns");
+          boolean linkSusi = _context.portMapper().isRegistered("susidns");
           long now = _context.clock().now();
           buf.append("<div class=\"leasesets_container\">");
           for (LeaseSet ls : leases) {
