@@ -72,6 +72,14 @@ public class Attachment {
 		return new FileInputStream(data);
 	}
 
+	/**
+	 * @return absolute path to the data file
+	 * @since 0.9.35
+	 */
+	public String getPath() {
+		return data.getAbsolutePath();
+	}
+
 	/**
 	 * The unencoded size
 	 * @since 0.9.33
