@@ -181,12 +181,8 @@ class EventPumper implements Runnable {
                 runDelayedEvents();
 
                 try {
-                    //if (_log.shouldLog(Log.DEBUG))
-                    //    _log.debug("before select...");
                     int count = _selector.select(SELECTOR_LOOP_DELAY);
                     if (count > 0) {
-                        //if (_log.shouldLog(Log.DEBUG))
-                        //    _log.debug("select returned " + count);
                         Set<SelectionKey> selected = _selector.selectedKeys();
                         //_context.statManager().addRateData("ntcp.pumperKeysPerLoop", selected.size());
                         processKeys(selected);
@@ -397,11 +393,9 @@ class EventPumper implements Runnable {
                     processConnect(key);
                 }
                 if (read) {
-                    //_context.statManager().addRateData("ntcp.read", 1, 0);
                     processRead(key);
                 }
                 if (write) {
-                    //_context.statManager().addRateData("ntcp.write", 1, 0);
                     processWrite(key);
                 }
                 //if (!(accept || connect || read || write)) {
@@ -414,24 +408,32 @@ class EventPumper implements Runnable {
             }
         }
     }
-    
+
     /**
      *  Called by the connection when it has data ready to write.
      *  If we have bandwidth, calls con.Write() which calls wantsWrite(con).
      *  If no bandwidth, calls con.queuedWrite().
      */
     public void wantsWrite(NTCPConnection con, byte data[]) {
-        ByteBuffer buf = ByteBuffer.wrap(data);
-        FIFOBandwidthLimiter.Request req = _context.bandwidthLimiter().requestOutbound(data.length, 0, "NTCP write");//con, buf);
+        wantsWrite(con, data, 0, data.length);
+    }
+
+    /**
+     *  Called by the connection when it has data ready to write.
+     *  If we have bandwidth, calls con.Write() which calls wantsWrite(con).
+     *  If no bandwidth, calls con.queuedWrite().
+     *
+     *  @since 0.9.35 off/len version
+     */
+    public void wantsWrite(NTCPConnection con, byte data[], int off, int len) {
+        ByteBuffer buf = ByteBuffer.wrap(data, off, len);
+        FIFOBandwidthLimiter.Request req = _context.bandwidthLimiter().requestOutbound(len, 0, "NTCP write");//con, buf);
         if (req.getPendingRequested() > 0) {
             if (_log.shouldLog(Log.INFO))
-                _log.info("queued write on " + con + " for " + data.length);
+                _log.info("queued write on " + con + " for " + len);
             _context.statManager().addRateData("ntcp.wantsQueuedWrite", 1);
             con.queuedWrite(buf, req);
         } else {
-            // fully allocated
-            //if (_log.shouldLog(Log.INFO))
-            //    _log.info("fully allocated write on " + con + " for " + data.length);
             con.write(buf);
         }
     }
@@ -475,12 +477,6 @@ class EventPumper implements Runnable {
             else
                 rv = ByteBuffer.allocate(BUF_SIZE);
             _numBufs++;
-            //if (_log.shouldLog(Log.DEBUG))
-            //    _log.debug("creating a new read buffer " + System.identityHashCode(rv) + " with " + __liveBufs + " live: " + rv);            
-            //_context.statManager().addRateData("ntcp.liveReadBufs", NUM_BUFS, 0);
-        } else {
-            //if (_log.shouldLog(Log.DEBUG))
-            //    _log.debug("acquiring existing read buffer " + System.identityHashCode(rv) + " with " + __liveBufs + " live: " + rv);
         }
         return rv;
     }
@@ -491,10 +487,6 @@ class EventPumper implements Runnable {
      *  High-frequency path in thread.
      */
     public static void releaseBuf(ByteBuffer buf) {
-        //if (false) return;
-        //if (_log.shouldLog(Log.DEBUG))
-        //    _log.debug("releasing read buffer " + System.identityHashCode(buf) + " with " + __liveBufs + " live: " + buf);
-
         // double check
         if (buf.capacity() < BUF_SIZE) {
             I2PAppContext.getGlobalContext().logManager().getLog(EventPumper.class).error("Bad size " + buf.capacity(), new Exception());
@@ -516,13 +508,9 @@ class EventPumper implements Runnable {
                 }
             }
         }
-        //if (cached && _log.shouldLog(Log.DEBUG))
-        //    _log.debug("read buffer " + System.identityHashCode(buf) + " cached with " + __liveBufs + " live");
     }
     
     private void processAccept(SelectionKey key) {
-        //if (_log.shouldLog(Log.DEBUG))
-        //    _log.debug("processing accept");
         ServerSocketChannel servChan = (ServerSocketChannel)key.attachment();
         try {
             SocketChannel chan = servChan.accept();
@@ -542,8 +530,6 @@ class EventPumper implements Runnable {
             if (_context.blocklist().isBlocklisted(ip)) {
                 if (_log.shouldLog(Log.WARN))
                     _log.warn("Receive session request from blocklisted IP: " + chan.socket().getInetAddress());
-                // need to add this stat first
-                // _context.statManager().addRateData("ntcp.connectBlocklisted", 1, 0);
                 try { chan.close(); } catch (IOException ioe) { }
                 return;
             }
@@ -559,14 +545,11 @@ class EventPumper implements Runnable {
                 return;
             }
 
-            // BUGFIX for firewalls. --Sponge
             if (shouldSetKeepAlive(chan))
                 chan.socket().setKeepAlive(true);
 
             SelectionKey ckey = chan.register(_selector, SelectionKey.OP_READ);
             new NTCPConnection(_context, _transport, chan, ckey);
-            //if (_log.shouldLog(Log.DEBUG))
-            //    _log.debug("new NTCP connection established: " +con);
         } catch (IOException ioe) {
             _log.error("Error accepting", ioe);
         }
@@ -580,7 +563,6 @@ class EventPumper implements Runnable {
             if (_log.shouldLog(Log.DEBUG))
                 _log.debug("processing connect for " + con + ": connected? " + connected);
             if (connected) {
-                // BUGFIX for firewalls. --Sponge
                 if (shouldSetKeepAlive(chan))
                     chan.socket().setKeepAlive(true);
                 con.setKey(key);
@@ -595,7 +577,6 @@ class EventPumper implements Runnable {
             if (_log.shouldLog(Log.INFO))
                 _log.info("Failed outbound " + con, ioe);
             con.closeOnTimeout("connect failed", ioe);
-            //_context.banlist().banlistRouter(con.getRemotePeer().calculateHash(), "Error connecting", NTCPTransport.STYLE);
             _transport.markUnreachable(con.getRemotePeer().calculateHash());
             _context.statManager().addRateData("ntcp.connectFailedTimeoutIOE", 1);
         } catch (NoConnectionPendingException ncpe) {
@@ -630,7 +611,6 @@ class EventPumper implements Runnable {
         try {
             int read = con.getChannel().read(buf);
             if (read < 0) {
-                //_context.statManager().addRateData("ntcp.readEOF", 1);
                 if (con.isInbound() && con.getMessagesReceived() <= 0) {
                     InetAddress addr = con.getChannel().socket().getInetAddress();
                     int count;
@@ -653,8 +633,6 @@ class EventPumper implements Runnable {
                 con.close();
                 releaseBuf(buf);
             } else if (read == 0) {
-                //if (_log.shouldLog(Log.DEBUG))
-                //    _log.debug("nothing to read for " + con + ", but stay interested");
                 // stay interested
                 //key.interestOps(key.interestOps() | SelectionKey.OP_READ);
                 releaseBuf(buf);
@@ -679,14 +657,9 @@ class EventPumper implements Runnable {
                 if (req.getPendingRequested() > 0) {
                     // rare since we generally don't throttle inbound
                     key.interestOps(key.interestOps() & ~SelectionKey.OP_READ);
-                    //if (_log.shouldLog(Log.DEBUG))
-                    //    _log.debug("bw throttled reading for " + con + ", so we don't want to read anymore");
                     _context.statManager().addRateData("ntcp.queuedRecv", read);
                     con.queuedRecv(buf, req);
                 } else {
-                    // fully allocated
-                    //if (_log.shouldLog(Log.DEBUG))
-                    //    _log.debug("not bw throttled reading for " + con);
                     // stay interested
                     //key.interestOps(key.interestOps() | SelectionKey.OP_READ);
                     con.recv(buf);
@@ -747,51 +720,31 @@ class EventPumper implements Runnable {
      *  High-frequency path in thread.
      */
     private void processWrite(SelectionKey key) {
-        //int totalWritten = 0;
-        //int buffers = 0;
-        //long before = System.currentTimeMillis();
         NTCPConnection con = (NTCPConnection)key.attachment();
         try {
             while (true) {
                 ByteBuffer buf = con.getNextWriteBuf();
                 if (buf != null) {
-                    //if (_log.shouldLog(Log.DEBUG))
-                    //    _log.debug("writing " + buf.remaining()+"...");
                     if (buf.remaining() <= 0) {
-                        //long beforeRem = System.currentTimeMillis();
                         con.removeWriteBuf(buf);
-                        //long afterRem = System.currentTimeMillis();
-                        //if (_log.shouldLog(Log.DEBUG))
-                        //    _log.debug("buffer was already fully written and removed after " + (afterRem-beforeRem) + "...");
-                        //buffers++;
                         continue;                    
                     }
                     int written = con.getChannel().write(buf);
                     //totalWritten += written;
                     if (written == 0) {
                         if ( (buf.remaining() > 0) || (!con.isWriteBufEmpty()) ) {
-                            //if (_log.shouldLog(Log.DEBUG)) _log.debug("done writing, but data remains...");
                             // stay interested
                             //key.interestOps(key.interestOps() | SelectionKey.OP_WRITE);
                         } else {
-                            //if (_log.shouldLog(Log.DEBUG)) _log.debug("done writing, no data remains...");
                             key.interestOps(key.interestOps() & ~SelectionKey.OP_WRITE);
                         }
                         break;
                     } else if (buf.remaining() > 0) {
-                        //if (_log.shouldLog(Log.DEBUG)) _log.debug("buffer data remaining...");
                         // stay interested
                         //key.interestOps(key.interestOps() | SelectionKey.OP_WRITE);
                         break;
                     } else {
-                        //long beforeRem = System.currentTimeMillis();
                         con.removeWriteBuf(buf);
-                        //long afterRem = System.currentTimeMillis();
-                        //if (_log.shouldLog(Log.DEBUG))
-                        //    _log.debug("buffer "+ buffers+"/"+written+"/"+totalWritten+" fully written after " +
-                        //               (beforeRem-before) + ", then removed after " + (afterRem-beforeRem) + "...");
-                        //releaseBuf(buf);
-                        //buffers++;
                         //if (buffer time is too much, add OP_WRITe to the interest ops and break?)
                         // LOOP
                     }
@@ -811,10 +764,6 @@ class EventPumper implements Runnable {
             _context.statManager().addRateData("ntcp.writeError", 1);
             con.close();
         }
-        //long after = System.currentTimeMillis();
-        //if (_log.shouldLog(Log.INFO))
-        //    _log.info("Wrote " + totalWritten + " in " + buffers + " buffers on " + con 
-        //              + " after " + (after-before));
     }
     
     /**
@@ -900,7 +849,6 @@ class EventPumper implements Runnable {
                     boolean connected = con.getChannel().connect(saddr);
                     if (connected) {
                         // Never happens, we use nonblocking
-                        //_context.statManager().addRateData("ntcp.connectImmediate", 1);
                         key.interestOps(SelectionKey.OP_READ);
                         processConnect(key);
                     }
@@ -909,24 +857,12 @@ class EventPumper implements Runnable {
                         _log.warn("error connecting to " + Addresses.toString(naddr.getIP(), naddr.getPort()), ioe);
                     _context.statManager().addRateData("ntcp.connectFailedIOE", 1);
                     _transport.markUnreachable(con.getRemotePeer().calculateHash());
-                    //if (ntcpOnly(con)) {
-                    //    _context.banlist().banlistRouter(con.getRemotePeer().calculateHash(), "unable to connect: " + ioe.getMessage());
-                    //    con.close(false);
-                    //} else {
-                    //    _context.banlist().banlistRouter(con.getRemotePeer().calculateHash(), "unable to connect: " + ioe.getMessage(), NTCPTransport.STYLE);
-                        con.close(true);
-                    //}
+                    con.close(true);
                 } catch (UnresolvedAddressException uae) {                    
                     if (_log.shouldLog(Log.WARN)) _log.warn("unresolved address connecting", uae);
                     _context.statManager().addRateData("ntcp.connectFailedUnresolved", 1);
                     _transport.markUnreachable(con.getRemotePeer().calculateHash());
-                    //if (ntcpOnly(con)) {
-                    //    _context.banlist().banlistRouter(con.getRemotePeer().calculateHash(), "unable to connect/resolve: " + uae.getMessage());
-                    //    con.close(false);
-                    //} else {
-                    //    _context.banlist().banlistRouter(con.getRemotePeer().calculateHash(), "unable to connect/resolve: " + uae.getMessage(), NTCPTransport.STYLE);
-                        con.close(true);
-                    //}
+                    con.close(true);
                 } catch (CancelledKeyException cke) {
                     con.close(false);
                 }
@@ -941,22 +877,7 @@ class EventPumper implements Runnable {
             _lastExpired = now;
         }
     }
-    
-    /**
-     * If the other peer only supports ntcp, we should banlist them when we can't reach 'em,
-     * but if they support other transports (eg ssu) we should allow those transports to be
-     * tried as well.
-     */
-/****
-    private boolean ntcpOnly(NTCPConnection con) {
-        RouterIdentity ident = con.getRemotePeer();
-        if (ident == null) return true;
-        RouterInfo info = _context.netDb().lookupRouterInfoLocally(ident.calculateHash());
-        if (info == null) return true;
-        return info.getAddresses().size() == 1;
-    }
-****/
-    
+
     private long _lastExpired;
 
     private void expireTimedOut() {
